
Imports System
Imports System.Drawing
Imports System.ComponentModel
Imports System.Collections
Imports Syncfusion.Windows.Forms.Grid
Imports System.Windows.Forms
Imports System.Data



'/ <summary>
'/ Helper class to faciliate multicolumn sorting 
'/ in a GridDataBoundGrid bound to a DataTable.
'/ </summary>
Public Class GridMultiColSortHelper

    '/ <summary>
    '/ the constructor
    '/ </summary>
    Public Sub New()
    End Sub 'New


    'tracks the columns that are currently sorted by position in sort.
    Private sortedCols As New ArrayList()

    'tracks the columns currently sorted by column name
    Private sortDirections As New SortedList()

    'indicates whether the helper class should autosize teh columns
    Private handleAutoSize As Boolean = False

    'a reference to the GridDataBoundGrid
    Private grid As GridDataBoundGrid = Nothing


    '/ <summary>
    '/ Sets teh helper class to manage sorting in the grid
    '/ </summary>
    '/ <param name="grid">the GridDataBoundGrid</param>
    '/ <param name="handleAutoSize">whether or not you want the helper class to resize the columns widths</param>
    Public Sub WireGrid(ByVal grid As GridDataBoundGrid, ByVal handleAutoSize As Boolean)
        If Not (Me.grid Is Nothing) AndAlso Not Me.grid.Equals(grid) Then
            UnwireGrid()
        End If
        If Not (Me.grid Is Nothing) AndAlso Me.grid.Equals(grid) Then
            Return
        End If
        Me.grid = grid

        AddHandler Me.grid.CellClick, AddressOf grid_CellClick
        AddHandler Me.grid.CellDoubleClick, AddressOf grid_CellDoubleClick
        AddHandler Me.grid.DrawCellDisplayText, AddressOf grid_DrawCellDisplayText
        Me.grid.BaseStylesMap("Column Header").StyleInfo.CellType = "Header"

        'test out the datasource
        Dim dv As DataView = GetDataView()

        If handleAutoSize Then
            'size columns so triangle will fit ok
            Me.grid.AllowResizeToFit = False
            Me.grid.Model.ColWidths.ResizeToFit(GridRangeInfo.Row(0))
            Dim i As Integer

            While i <= Me.grid.Model.ColCount
                Me.grid.Model.ColWidths(i) = Me.grid.Model.ColWidths(i) + 12
                i += 1
            End While
        End If
        Me.handleAutoSize = handleAutoSize
    End Sub 'WireGrid


    '/ <summary>
    '/ unhooks the helper object from the GridDataBoundGrid
    '/ </summary>
    Public Sub UnwireGrid()
        If Me.grid Is Nothing Then
            Return
        End If
        RemoveHandler Me.grid.CellClick, AddressOf grid_CellClick
        RemoveHandler Me.grid.CellDoubleClick, AddressOf grid_CellDoubleClick
        RemoveHandler Me.grid.DrawCellDisplayText, AddressOf grid_DrawCellDisplayText
        Me.grid.BaseStylesMap("Column Header").StyleInfo.CellType = "SortColumnHeader"

        Me.grid = Nothing
    End Sub 'UnwireGrid


    'gets at the DataView
    Private Function GetDataView() As DataView
        Dim cm As CurrencyManager = CType(Me.grid.BindingContext(Me.grid.DataSource, Me.grid.DataMember), CurrencyManager)

        If Not (cm Is Nothing) Then
            Dim dv As DataView = CType(cm.List, DataView)

            If Not (dv Is Nothing) Then
                Return dv
            End If
        End If
        Throw New ArgumentException("DataSource must be a DataTable or DataView")
    End Function 'GetDataView


    'draws the cell text with sort icon if needed
    Private Sub grid_DrawCellDisplayText(ByVal sender As Object, ByVal e As GridDrawCellDisplayTextEventArgs)
        If e.Style.CellIdentity.ColIndex > 0 AndAlso e.Style.CellIdentity.RowIndex = 0 AndAlso Me.sortDirections.Count > 0 Then
            Dim field As Integer = Me.grid.Binder.ColIndexToField(e.Style.CellIdentity.ColIndex)
            Dim gbcc As GridBoundColumnsCollection = IIf(Me.grid.GridBoundColumns Is Nothing OrElse Me.grid.GridBoundColumns.Count = 0, Me.grid.Binder.InternalColumns, Me.grid.GridBoundColumns)
            Dim colName As String = gbcc(field).MappingName
            If Not (Me.sortDirections(colName) Is Nothing) Then
                Dim listSortDirection As ListSortDirection = CType(Me.sortDirections(colName), ListSortDirection)
                Dim margin As Integer = 12
                Dim textRectangle As Rectangle = e.TextRectangle
                textRectangle.Width -= margin

                GridStaticCellRenderer.DrawText(e.Graphics, e.DisplayText, e.Style.GdipFont, textRectangle, e.Style, e.Style.TextColor, False)


                Dim rect As New Rectangle(textRectangle.Right, textRectangle.Y, 10, textRectangle.Height)
                rect = GridUtil.CenterInRect(rect, New Size(8, 8))

                Dim brush = New SolidBrush(SystemColors.ControlDark)

                'g.FillRectangle(brush, rect);
                Dim i2 As Integer = Math.Max(0, (rect.Height - 3) / 2)
                rect.Inflate(-i2, -i2)
                Dim pen1 As New Pen(SystemColors.WindowFrame)
                Dim pen2 As New Pen(SystemColors.Control)
                Dim triangleDirection As GridTriangleDirection = IIf(listSortDirection = listSortDirection.Ascending, GridTriangleDirection.Up, GridTriangleDirection.Down)
                GridPaintTriangle.Paint(e.Graphics, rect, triangleDirection, brush, pen1, True)
                pen1.Dispose()
                pen2.Dispose()

                e.Cancel = True
            End If
        End If
    End Sub 'grid_DrawCellDisplayText


    'handles the click sorting
    Private Sub grid_CellClick(ByVal sender As Object, ByVal e As GridCellClickEventArgs)
        If Me.grid.SortBehavior <> GridSortBehavior.SingleClick Then
            Return
        End If
        If e.RowIndex = 0 Then
            HandleSortCol(e.ColIndex)
            e.Cancel = True
        End If
    End Sub 'grid_CellClick


    'handles the double click sorting
    Private Sub grid_CellDoubleClick(ByVal sender As Object, ByVal e As GridCellClickEventArgs)
        If Me.grid.SortBehavior <> GridSortBehavior.DoubleClick Then
            Return
        End If
        If e.RowIndex = 0 Then
            HandleSortCol(e.ColIndex)
            e.Cancel = True
        End If
    End Sub 'grid_CellDoubleClick


    'sets up the sort information in sortDirections and sortedCols.
    Private Sub HandleSortCol(ByVal colIndex As Integer)
        Dim field As Integer = Me.grid.Binder.ColIndexToField(colIndex)
        If field > -1 Then
            Dim gbcc As GridBoundColumnsCollection = IIf(Me.grid.GridBoundColumns Is Nothing OrElse Me.grid.GridBoundColumns.Count = 0, Me.grid.Binder.InternalColumns, Me.grid.GridBoundColumns)
            Dim colName As String = gbcc(field).MappingName
            Dim dir As ListSortDirection = ListSortDirection.Ascending

            If Not (Me.sortDirections(colName) Is Nothing) Then
                dir = IIf(CType(Me.sortDirections(colName), ListSortDirection) = ListSortDirection.Ascending, ListSortDirection.Descending, ListSortDirection.Ascending)
            End If

            If (Control.ModifierKeys And Keys.Control) = 0 Then
                Me.sortedCols.Clear()
                Me.sortDirections.Clear()
            End If

            If Not (Me.sortDirections(colName) Is Nothing) Then
                Me.sortDirections(colName) = dir
            Else
                Me.sortedCols.Add(colName)
                Me.sortDirections.Add(colName, dir)
            End If

            SortColumns()
        End If
    End Sub 'HandleSortCol


    '/ <summary>
    '/ Lets you programaticcally sort multiple columns
    '/ </summary>
    '/ <param name="colName">name of column to be sorted</param>
    '/ <param name="dir">the direction of the sort</param>
    '/ <param name="clearSortedColumns">whether previous sorts should be cleared</param>
    '/ <param name="initiateSort">whether the sort should be done.</param>
    Public Sub SortColumn(ByVal colName As String, ByVal dir As ListSortDirection, ByVal clearSortedColumns As Boolean, ByVal initiateSort As Boolean)
        If clearSortedColumns Then
            Me.sortedCols.Clear()
            Me.sortDirections.Clear()
        End If

        If Not (Me.sortDirections(colName) Is Nothing) Then
            Me.sortDirections(colName) = dir
        Else
            Me.sortedCols.Add(colName)
            Me.sortDirections.Add(colName, dir)
        End If

        If initiateSort Then
            SortColumns()
        End If
    End Sub 'SortColumn

    'sorts the columns as specified in sortedCols and sortDirections through the DataView
    Private Sub SortColumns()
        Dim sort As String = ""
        Dim i As Integer

        While i < Me.sortedCols.Count
            Dim colName As String = Me.sortedCols(i).ToString()
            Dim dir As String = IIf(Me.sortDirections(colName).Equals(ListSortDirection.Ascending), "Asc", "Desc")
            If i > 0 Then
                sort += ","c
            End If
            sort += String.Format("[{0}] {1}", colName, dir)
            Dim colIndex As Integer = Me.grid.Binder.NameToColIndex(colName)

            Me.grid(0, colIndex).Tag = Me.sortDirections(colName)
            i += 1
        End While
        'Console.WriteLine(sort);
        Dim dv As DataView = GetDataView()
        dv.Sort = sort
    End Sub 'SortColumns 
End Class 'GridMultiColSortHelper

